pub static LOCALES: &[(&str, &str)] = &[
    ("de", include_str!("/build/numa/src/numa-0.23.0/site/locales/de.json")),
    ("en", include_str!("/build/numa/src/numa-0.23.0/site/locales/en.json")),
    ("manifest", include_str!("/build/numa/src/numa-0.23.0/site/locales/manifest.json")),
    ("zh-cn", include_str!("/build/numa/src/numa-0.23.0/site/locales/zh-cn.json")),
];
